/*
    Yawn game
    Copyright (c) 1998-2000,2001 by Alessandro Scotti
*/
package org.ascotti.yawn;

import java.applet.Applet;
import java.lang.Exception;
import java.lang.Math;
import java.util.Random;
import java.awt.*;
import java.awt.event.*;

import org.ascotti.yawn.*;

class YawnScoreSheet
{
    public int [] cell;
    public Label [] label;

    static final int Tris = 6;
    static final int Poker = 7;
    static final int Full = 8;
    static final int SmallStraight = 9;
    static final int Straight = 10;
    static final int Yawn = 11;
    static final int Chance = 12;

    public YawnScoreSheet()
    {
        int i;

        cell = new int [13];
        label = new Label [13];
        for( i=0; i<13; i++ ) label[i] = new Label();
    }

    public void reset()
    {
        int i;

        for( i=0; i<13; i++ ) {
            cell[i] = 0;
            label[i].setText( "" );
        }
    }

    public void recalc( YawnDice [] dice )
    {
        int i, straightMask = 0, gotTris = 0;

        for( i = 0; i < 13; i++ )   // Clear score sheet
            cell[i] = 0;

        for( i = 0; i < dice.length; i++ )    // Get dice sum ("chance")
            cell[Chance] += dice[i].dice.getValue();

        for( i = 0; i < dice.length; i++ ) {
            int currentDice = dice[i].dice.getValue();
            int currentCell = cell[ currentDice-1 ];

            currentCell += currentDice;

            if( currentCell == currentDice*3 )  // Tris
                gotTris = currentDice;
            if( currentCell == currentDice*4 )  // Poker
                cell[Poker] = cell[Chance];
            if( currentCell == currentDice*5 )  // Yawn
                cell[Yawn] = 50;

            cell[ currentDice-1 ] = currentCell;

            straightMask |= 1 << (currentDice-1);
        }

        if( gotTris != 0 ) {    // Three of a kind, check for full house
            cell[Tris] = cell[Chance];
            for( i = 0; i < 6; i++ )
                if( cell[i] == 2*(i+1) || cell[Yawn] != 0 )
                    cell[Full] = 25;
        }

        if( ((straightMask & 0x0F) == 0x0F) || ((straightMask & 0x1E) == 0x1E) || ((straightMask & 0x3C) == 0x3C) )
            cell[SmallStraight] = 30;
        if( ((straightMask & 0x1F) == 0x1F) || ((straightMask & 0x3E) == 0x3E) )
            cell[Straight] = 40;
    }
}

class YawnDice {
    public Dice dice;
    public boolean locked;
    public Label label;

    public YawnDice() {
        locked = false;
        dice = new Dice();
        label = new Label("");
        label.setFont( new Font( "Courier", Font.BOLD, 13 ) );
    }

    public void roll() { if( !locked ) dice.doRoll(); }
    public void toggle() { locked = !locked; label.setText( locked ? "Hold" : "" ); }
    public void reset() { locked = false; label.setText( "" ); }
    public void hide() { dice.setVisible( false ); label.setVisible( false ); }
    public void show() { dice.setVisible( true ); label.setVisible( true ); }
    public void set( int v ) { if( !locked ) dice.setValue( v ); }
}

class YawnScorePanel extends Panel
{
    private void frameRect( Graphics g, int x, int y, int width, int height )
    {
        g.setColor( getBackground() );
        g.fillRect( x, y, width, height );
        g.setColor( getForeground() );
        g.drawRect( x, y, width, height );
    }

    public void paint( Graphics g )
    {
        int     ox = 0, oy = 0, ow = getSize().width-1, oh = getSize().height-1;
        int     i, x, y;

        g.setColor( getForeground() );
        g.drawRect( ox, oy, ow, oh );
        oh = (oh - 2) / 7;
        ow = (ow / 2) - 1;
        ox += 1;
        for( i = 0; i <= 6; i++ ) {
            y = oy + oh * i + 1;
            frameRect( g, ox, y, ow, oh );
            frameRect( g, ox+ow+1, y, ow, oh );
        }
    }
}

/*
*/
public class Yawn extends Applet
{
    YawnScoreSheet  scoreSheet;
    YawnScorePanel  scorePanel;
    BitBtn          rollButton;
    BitBtn []       diceButton;
    Label           scoreLabel;
    Panel           fDicePanel;
    YawnDice []     fDice;
    int             fRollsLeft = 3;
    int             fShotsLeft = 13;
    Random          fRandomizer;

    private BitBtn makeButton( String aLabel, String aImageName, int x, int y, int width, int height )
    {
        BitBtn  btn;

        btn = new BitBtn( aLabel, getImage(getClass().getResource(aImageName+".gif")) );
        btn.setBackground( SystemColor.control );
        btn.setBounds( x, y, width, height );
        btn.setMargin( 6 );
        return btn;
    }

    public void init()
    {
        BitBtn  btn;
        int     i;

        /* Create randomizer */
        fRandomizer = new Random();

        /* */
        setLayout( null );
        setBackground( SystemColor.control );

        /* Start button */
        btn = makeButton( "Restart", "r", 233, 370, 98, 33 );
        btn.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                int j;

                for( j=0; j<diceButton.length; j++ )
                    diceButton[j].setEnabled( true );
                scoreSheet.reset();
                for( j=0; j<5; j++ ) {
                    fDice[j].hide();
                    fDice[j].reset();
                }
                fRollsLeft = 3;
                fShotsLeft = 13;
                rollButton.setEnabled( true );
                updateScore( 0 );
            }
        });
        add( btn );

        /* Roll button */
        rollButton = makeButton( "Roll", "r", 7, 304, 58, 57 );
        rollButton.setEnabled( false );
        rollButton.setStyle( BitBtn.TOP );
        rollButton.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                YawnScoreSheet  s = new YawnScoreSheet();
                int             j;

                for( j=0; j<fDice.length; j++ ) {
                    fDice[j].set( Math.abs(1 + (fRandomizer.nextInt() % 6)) );
                    fDice[j].show();
                }

                s.recalc( fDice );
                for( j=0; j<diceButton.length; j++ )
                    if( diceButton[j].isEnabled() ) {
                        scoreSheet.label[j].setForeground( Color.gray );
                        scoreSheet.label[j].setText( "" + s.cell[j] );
                        scoreSheet.cell[j] = s.cell[j];
                    }
                s = null;

                if( --fRollsLeft == 0 )
                    rollButton.setEnabled( false );
            }
        });
        add( rollButton );

        /* Score label */
        scoreLabel = new Label( "" );
        scoreLabel.setFont( new Font( "SansSerif", Font.BOLD | Font.ITALIC, 24 ) );
        scoreLabel.setBounds( 7, 370, 200, 30 );
        updateScore( 0 );
        add( scoreLabel );

        /* */
        int [] lblLeft = { 120,120,120,120,120,120,176,176,176,176,176,176,176 };
        int [] lblTop = { 24,64,104,144,184,224,24,64,104,144,184,224,264 };
        scoreSheet = new YawnScoreSheet();
        for( i=0; i<scoreSheet.label.length; i++ ) {
            scoreSheet.label[i].setBounds( lblLeft[i], lblTop[i]-2,  42, 22 );
            scoreSheet.label[i].setAlignment( Label.RIGHT );
            scoreSheet.label[i].setBackground( Color.white );
            scoreSheet.label[i].setFont( new Font( "SansSerif", Font.BOLD, 20 ) );
            add( scoreSheet.label[i] );
        }

        /* Score panel */
        scorePanel = new YawnScorePanel();
        scorePanel.setBackground( Color.white );
        scorePanel.setForeground( Color.blue );
        scorePanel.setBounds( 112, 8, 114, 290 );
        add( scorePanel );

        /* Cell buttons */
        int [] btnLeft = { 8,8,8,8,8,8,233,233,233,233,233,233,233 };
        int [] btnTop = { 16,56,96,136,176,216,16,56,96,136,176,216,256 };
        int [] btnWidth = { 98,98,98,98,98,98,98,98,98,98,98,98,98 };
        int [] btnHeight = { 33,33,33,33,33,33,33,33,33,33,33,33,33 };
        String [] btnLabel = { "Ace","Two","Three","Four","Five","Six","Tris","Poker","Full","Small","Straight","Yawn","Chance" };
        String [] btnImage = { "1","2","3","4","5","6","t","p","f","s4","s5","y","c" };

        diceButton = new BitBtn [ btnLeft.length ];
        for( i=0; i<btnLeft.length; i++ ) {
            btn = makeButton( btnLabel[i], btnImage[i], btnLeft[i], btnTop[i], btnWidth[i], btnHeight[i] );
            if( i > 5 )
                btn.setStyle( BitBtn.RIGHT );
            btn.setTag( i );
            btn.addActionListener( new ActionListener() {
                public void actionPerformed( ActionEvent e ) {
                    BitBtn b = (BitBtn)(e.getSource());
                    int tag = b.getTag();

                    if( fRollsLeft == 3 )
                        return;
                    scoreSheet.label[tag].setForeground( Color.black );
                    b.setEnabled( false );
                    fRollsLeft = 3;
                    for( tag=0; tag<5; tag++ ) {
                        fDice[tag].hide();
                        fDice[tag].reset();
                    }
                    if( --fShotsLeft > 0 ) {
                        rollButton.setEnabled( true );
                        for( tag=0; tag<scoreSheet.label.length; tag++ ) {
                            if( diceButton[tag].isEnabled() )
                                scoreSheet.label[tag].setText( "" );
                        }
                    }
                    else {
                        rollButton.setEnabled( false );
                    }
                    updateScore( -1 );
                }
            });
            add( btn );
            diceButton[i] = btn;
        }

        /* Dice panel */
        int [] dicLeft = { 16, 64, 112, 160, 208 };

        fDicePanel = new Panel();
        fDicePanel.setBackground( Color.green.darker() );
        fDicePanel.setBounds( 73, 304, 257, 57 );
        fDicePanel.setLayout( null );
        fDice = new YawnDice [5];
        for( i=0; i<5; i++ ) {
            fDice[i] = new YawnDice();
            fDice[i].hide();
            fDice[i].dice.setBackground( Color.white );
            fDice[i].dice.setLocation( dicLeft[i], 5 );
            fDice[i].dice.setTag( i );
            fDice[i].dice.addActionListener( new ActionListener() {
                public void actionPerformed( ActionEvent e ) {
                    Dice dice = (Dice)(e.getSource());
                    fDice[dice.getTag()].toggle();
                }
            });
            fDice[i].label.setBackground( fDicePanel.getBackground() );
            fDice[i].label.setBounds( dicLeft[i]-2, 39, 34, 16 );
            fDicePanel.add( fDice[i].dice );
            fDicePanel.add( fDice[i].label );
        }
        add( fDicePanel );
    }

    protected void updateScore( int aScore )
    {
        int i, bonus = 0;

        if( aScore == -1 ) {
            aScore = 0;
            for( i=0; i<scoreSheet.cell.length; i++ )
                aScore += scoreSheet.cell[i];
            for( i=0; i<6; i++ )
                bonus += scoreSheet.cell[i] - (i+1)*3;
            if( fShotsLeft == 0 ) {
                bonus = (bonus >= 0 ? 35 : 0);
                aScore += bonus;
            }
        }
        scoreLabel.setText( "Your score: " + aScore );
    }
}
